//*****************************************
// RTC & 7seg LED Program Mega88
// CPU Clock:8MHz
//   iq[Yf[^@UWlgɂj
// File Name:Rtc_DS3234.c
//
//  PB2(SS)  : SPI-SS
//  PB3(MOSI): SPI-MOSI(Mega88:out, DS3234:in)
//  PB4(MISO): SPI-MISO(Mega88:in,  DS3234:out)
//  PB5(SCT) : SPI-SCK
//
//  PD2(INT0): INT0(interrupt from DS3234)
//
//  C:\WinAVR-20100110\doc\avr-libc\avr-libc-user-manual.pdf ɊeCN[ht@C̐
//
//  2012/06/13 New   (for DS3231)
//  2014/02/17 modify(for DS3234)
//*****************************************

#include	<avr/interrupt.h>
#include	<avr/io.h>
#include	<avr/pgmspace.h>

#include	"RTC_7SEG.h"
#include	"Mega_Spi.h"
#include	"Timer.h"
#include	"disp_7seg_led.h"
#include	"port_bit.h"

#include	"Rtc_DS3234.h"


unsigned char	gRTC_sec;				// qsb b̃f[^ (BCD 0--59)
unsigned char	gRTC_min;				// qsb ̃f[^ (BCD 0--59)
unsigned char	gRTC_hou;				// qsb ̃f[^ (BCD 0--23j
unsigned char	gRTC_wday;				// qsb j̃f[^ (BCD 1--7)
unsigned char	gRTC_day;				// qsb ̃f[^ (BCD 1--31)
unsigned char	gRTC_mon;				// qsb ̃f[^ (BCD 1--12j
unsigned char	gRTC_year;				// qsb Ñf[^ (BCD 0--99)
unsigned char	gRTC_control;			// qsb Rg[̃f[^
unsigned char	gRTC_status;			// qsb XeC^X̃f[^
unsigned char	gCount_sqw;				// rpvM̃JEg
unsigned int	gTemperature;			// qsb xf[^iPOO{̒l 3075 : 30.75xj

unsigned char	rtc_1s_flg;				// qsbĂ鎞̂Pboߗp

//***************************
// RTC Initial
//  qsb̏ݒ
//***************************
void	rtc_init(void)
{
	unsigned char	t;

	spi_init();

	// Pbo͐ݒ
	//  PD2(INT0): Pulse In from RTC Square Wave Signal(1Hz,4kHz,8kHz,32kHz)

	DDRD &= ~(1 << DDD2);		// Port-D Bit2 : Intput
	PORTD |= (1 << PD2);		// PD2 Pullup

	rtc_status_read();			// RTC XeC^Xf[^ǂݏo
	rtc_control_read();			// RTC Rg[f[^ǂݏo

	t = gRTC_control;			// ǂݏoRg[f[^ꎞޔ
	gRTC_control = 0x00;		// EOSC:0, BBSQW:0, CONV:0, RS2:0 RS1:0(1Hz) INTCN:0(SQUARE-WAVE OUTPUT), A2IE:0, A1IE=0
	rtc_control_write();		// RTC Rg[f[^
	gRTC_control = t;			// ꎞޔf[^߂

	// INT0 Interrupt Initial
	EICRA |=  (1 << ISC01);		// INT0 Falling Edge Interrupt
	EICRA &= ~(1 << ISC00);		// INT0 Falling Edge Interrupt
	EIMSK |= (1 << INT0);		// INT0 Interrupt Enable

}

//***************************
// RTC ǂݏof[^NA
//***************************
void	rtc_read_data_clear(void)
{
	gRTC_sec = 0;			// qsb b̃f[^ (BCD 0--59)
	gRTC_min = 0;			// qsb ̃f[^ (BCD 0--59)
	gRTC_hou = 0;			// qsb ̃f[^ (BCD 0--23j
	gRTC_wday = 0;			// qsb j̃f[^ (BCD 1--7)
	gRTC_day = 0;			// qsb ̃f[^ (BCD 1--31)
	gRTC_mon = 0;			// qsb ̃f[^ (BCD 1--12j
	gRTC_year = 0;			// qsb Ñf[^ (BCD 0--99)
}

//***************************
// RTC ǂݏo
//  ǂݏof[^͈ȉ̕ϐɓ
//	unsigned char	gRTC_sec;	// qsb b̃f[^ (BCD 0--59)
//	unsigned char	gRTC_min;	// qsb ̃f[^ (BCD 0--59)
//	unsigned char	gRTC_hou;	// qsb ̃f[^ (BCD 0--23j
//	unsigned char	gRTC_day;	// qsb j̃f[^ (BCD 1--7)
//	unsigned char	gRTC_date;	// qsb ̃f[^ (BCD 1--31)
//	unsigned char	gRTC_mon;	// qsb ̃f[^ (BCD 1--12j
//	unsigned char	gRTC_year;	// qsb Ñf[^ (BCD 0--99)
//***************************
void rtc_read(void)
{
	unsigned char	data;
	unsigned char	dumy;

	spi_start();

	dumy = spi_rd_wr(0x00);		// SPI  AhX

	data = spi_rd_wr(dumy);		// SPI ǂݏo
	gRTC_sec = data;			// qsb b̃f[^ (BCD 0--59)

	data = spi_rd_wr(dumy);		// SPI ǂݏo
	gRTC_min = data;			// qsb ̃f[^ (BCD 0--59)

	data = spi_rd_wr(dumy);		// SPI ǂݏo
	gRTC_hou = data;			// qsb ̃f[^ (BCD 0--23j

	data = spi_rd_wr(dumy);		// SPI ǂݏo
	gRTC_wday = data;			// qsb j̃f[^ (BCD 1--7)

	data = spi_rd_wr(dumy);		// SPI ǂݏo
	gRTC_day = data;			// qsb ̃f[^ (BCD 1--31)

	data = spi_rd_wr(dumy);		// SPI ǂݏo
	gRTC_mon = data;			// qsb ̃f[^ (BCD 1--12j

	data = spi_rd_wr(dumy);		// SPI ǂݏo
	gRTC_year = data;			// qsb Ñf[^ (BCD 0--99)

	spi_stop();

}

//***************************
// RTC 
//  ݃f[^͈ȉ̕ϐɓĂ镨Ƃ
//	unsigned char	gRTC_sec;	// qsb b̃f[^ (BCD 0--59)
//	unsigned char	gRTC_min;	// qsb ̃f[^ (BCD 0--59)
//	unsigned char	gRTC_hou;	// qsb ̃f[^ (BCD 0--23j
//	unsigned char	gRTC_wday;	// qsb j̃f[^ (BCD 1--7)
//	unsigned char	gRTC_day;	// qsb ̃f[^ (BCD 1--31)
//	unsigned char	gRTC_mon;	// qsb ̃f[^ (BCD 1--12j
//	unsigned char	gRTC_year;	// qsb Ñf[^ (BCD 0--99)
//***************************
void	rtc_write(void)
{
	unsigned char	dumy;

	gRTC_sec &= 0x7F;
	spi_start();						// SPI Jn

	dumy=spi_rd_wr(0x00 | 0x80);		// SPI  AhX

	dumy=spi_rd_wr(gRTC_sec);			// b̃f[^ (BCD 0--59)
	dumy=spi_rd_wr(gRTC_min);			// ̃f[^ (BCD 0--59)
	dumy=spi_rd_wr(gRTC_hou);			// ̃f[^ (BCD 0--23j
	dumy=spi_rd_wr(gRTC_wday);			// j̃f[^ (BCD 1--7)
	dumy=spi_rd_wr(gRTC_day);			// ̃f[^ (BCD 1--31)
	dumy=spi_rd_wr(gRTC_mon);			// ̃f[^ (BCD 1--12j
	dumy=spi_rd_wr(gRTC_year);			// Ñf[^ (BCD 0--99)

	spi_stop();

}

//***************************
// RTC Rg[f[^ǂݏo
//  ǂݏof[^ gRTC_control ɓ
//
//
//***************************
void	rtc_control_read(void)
{
	unsigned char	data;
	unsigned char	dumy;

	spi_start();									// SPI Jn

	dumy = spi_rd_wr(RTC_CONTROL_ADDR);				// SPI  AhX

	data = spi_rd_wr(dumy);							// SPI ǂݏo
	gRTC_control = data;							// qsb Rg[̃f[^

	spi_stop();

}
//***************************
// RTC Rg[f[^
//  ݃f[^ unsigned char gRTC_control ɓĂ镨Ƃ
//***************************
void	rtc_control_write(void)
{
	unsigned char	dumy;

	spi_start();									// SPI Jn

	dumy = spi_rd_wr(RTC_CONTROL_ADDR | 0x80);			// SPI  AhX
	dumy = spi_rd_wr(gRTC_control);					// Rg[f[^

	spi_stop();

}

//***************************
// RTC XeC^Xf[^ǂݏo
//  ǂݏof[^ gRTC_status ɓ
//
//
//***************************
void	rtc_status_read(void)
{
	unsigned char	data=0;
	unsigned char	dumy;

	spi_start();									// SPI Jn

	dumy = spi_rd_wr(RTC_SATUS_ADDR);				// SPI  AhX

	data = spi_rd_wr(dumy);							// SPI ǂݏo Ō
	gRTC_status = data;								// qsb XeC^X̃f[^

	spi_stop();

}
//***************************
// RTC XeC^Xf[^
//  ݃f[^ unsigned char gRTC_status ɓĂ镨Ƃ
//***************************
void	rtc_status_write(void)
{
	unsigned char	dumy;

	spi_start();									// SPI Jn

	dumy = spi_rd_wr(RTC_SATUS_ADDR | 0x80);		// SPI  AhX
	dumy = spi_rd_wr(gRTC_status);					// XeC^Xf[^

	spi_stop();

}
//*****************************************
// xANCWJn
//*****************************************
void	temperature_start(void)
{
	gRTC_control |= RTC_CONTROL_CONV;	// CONV:1
	rtc_control_write();				// RTC Rg[f[^

}
//*****************************************
// xǂݏo
//@ǂݏof[^
//   gTemperature_upgTemperature_dw ɓ
//  ANCWIĂȂ́@Ȃ
//*****************************************
void	temperature_read(void)
{
	unsigned char	data_u;
	unsigned char	data_l;
	unsigned char	dumy;

	rtc_control_read();								// RTC Rg[f[^ǂݏo

	if(	(gRTC_control & RTC_CONTROL_CONV) != 0){	// CONV==1 ANCWIĂȂ
		return;										// xǂ܂ȂŏI
	}

	// xǂݍ
	spi_start();									// SPI Jn

	dumy = spi_rd_wr(RTC_TEMPE_UP_ADDR);			// SPI ݃AhXixʃf[^j
	data_u = spi_rd_wr(dumy);						// SPI ǂݏo
	data_l = spi_rd_wr(dumy);						// SPI ǂݏo

	spi_stop();

	gTemperature = (data_u & ~0x80);
	gTemperature *= 100;
	if((data_l & 0x80)!=0){
		gTemperature += 50;
	}
	if((data_l & 0x40)!=0){
		gTemperature += 25;
	}

}
//*****************************************
// N킹
// ߂l@OFɂqsbɃANZXł
// @@@@PFɂqsbɃANZXłȂ
// @OFԑ얳
//*****************************************
unsigned char	proc_set_year(void)
{
	unsigned char	blk_stat=0;
	unsigned char	ye;
	unsigned char	ucr;
	unsigned char	r=0;

	ye = gRTC_year;
	set_7seg_data(2,0,0);
	set_7seg_data(0,0,1);
	set_7seg_data(ye >> 4,0,2);
	set_7seg_data(ye & 0x0f,0,3);

	tim_10ms_sblink = 0;						// _
	while(1){
		if(tim_10ms_sblink == 0){				// _Ń^C~O̎
			tim_10ms_sblink = 50;
			if(blk_stat==0){					// ł̎
				blk_stat = 1;
				set_7seg_data(16,0,2);
				set_7seg_data(16,0,3);
			}else{
				blk_stat = 0;
				set_7seg_data(ye >> 4,0,2);
				set_7seg_data(ye & 0x0f,0,3);
			}
		}

		ucr = get_action_sw();					// XCb`Ԃ
		ucr = ucr & 0x0f;						// EXCb`ɂ
		if(ucr != 0){							// EANVL鎞
			proc_no_key_timeup(1);				// L[͂L
			if(ucr == SWR_ON_CONTINUE){			// ꑱ
				ye++;
				if(9 < (ye & 0x0f)){			// X𒴂
					ye &= 0xf0;
					ye += 0x10;
				}
				if(0x99 < ye){
					ye = 0;
				}
			}else if(ucr == SWR_OFF_SHORT){		// Zꂽ
				if(gRTC_year == ye){			// lύXĂȂ
					r = 0;
				}else{
					gRTC_year = ye;
					rtc_write();				// RTC 
				}
				break;
			}
		}

		if (proc_no_key_timeup(0) == 0xff){		// ԃL[͂
			r = 0xff;
			break;
		}
	}

	return(r);
}
//*****************************************
// 킹
// ߂l@OFɂqsbɃANZXł
// @@@@PFɂqsbɃANZXłȂ
// @OFԑ얳
//*****************************************
unsigned char	proc_set_month(void)
{
	unsigned char	blk_stat=0;
	unsigned char	mo;
	unsigned char	ucr;
	unsigned char	r=0;

	mo = gRTC_mon;
	set_7seg_data(mo >> 4,1,0);
	set_7seg_data(mo & 0x0f,0,1);
	set_7seg_data(gRTC_day >> 4,0,2);
	set_7seg_data(gRTC_day & 0x0f,0,3);

	tim_10ms_sblink = 0;						// _
	while(1){
		if(tim_10ms_sblink == 0){				// _Ń^C~O̎
			tim_10ms_sblink = 50;
			if(blk_stat==0){					// ł̎
				blk_stat = 1;
				set_7seg_data(16,1,0);
				set_7seg_data(16,0,1);
			}else{
				blk_stat = 0;
				set_7seg_data(mo >> 4,1,0);
				set_7seg_data(mo & 0x0f,0,1);
			}
		}

		ucr = get_action_sw();					// XCb`Ԃ
		ucr = ucr & 0x0f;						// EXCb`ɂ
		if(ucr != 0){							// EANVL鎞
			proc_no_key_timeup(1);				// L[͂L
			if(ucr == SWR_ON_CONTINUE){			// ꑱ
				mo++;
				if(9 < (mo & 0x0f)){			// X𒴂
					mo &= 0xf0;
					mo += 0x10;
				}
				if(0x12 < mo){
					mo = 1;
				}
			}else if(ucr == SWR_OFF_SHORT){		// Zꂽ
				if(gRTC_mon == mo){				// lύXĂȂ
					r = 0;
				}else{
					gRTC_mon = mo;
					rtc_write();				// RTC 
				}
				break;
			}
		}

		if (proc_no_key_timeup(0) == 0xff){		// ԃL[͂
			r = 0xff;
			break;
		}
	}

	return(r);
}
//*****************************************
// 킹
// ߂l@OFɂqsbɃANZXł
// @@@@PFɂqsbɃANZXłȂ
// @OFԑ얳
//*****************************************
unsigned char	proc_set_day(void)
{
	unsigned char	blk_stat=0;
	unsigned char	da;
	unsigned char	ucr;
	unsigned char	r=0;
	unsigned char	uru=0;
	unsigned char	syo=0;

	if(((((gRTC_year/16)*10)+(gRTC_year/16))/4) == 0){									// 邤N
		uru=1;
	}
	if((gRTC_mon == 0x04)||(gRTC_mon == 0x06)||(gRTC_mon == 0x09)||(gRTC_mon == 0x11)){	// ̌
		syo=1;
	}
	da = gRTC_day;
	set_7seg_data(gRTC_mon >> 4,0,0);
	set_7seg_data(gRTC_mon & 0x0f,1,1);
	set_7seg_data(da >> 4,0,2);
	set_7seg_data(da & 0x0f,0,3);


	tim_10ms_sblink = 0;						// _
	while(1){
		if(tim_10ms_sblink == 0){				// _Ń^C~O̎
			tim_10ms_sblink = 50;
			if(blk_stat==0){					// ł̎
				blk_stat = 1;
				set_7seg_data(16,0,2);
				set_7seg_data(16,0,3);
			}else{
				blk_stat = 0;
				set_7seg_data(da >> 4,0,2);
				set_7seg_data(da & 0x0f,0,3);
			}
		}

		ucr = get_action_sw();					// XCb`Ԃ
		ucr = ucr & 0x0f;						// EXCb`ɂ
		if(ucr != 0){							// EANVL鎞
			proc_no_key_timeup(1);				// L[͂L
			if(ucr == SWR_ON_CONTINUE){			// ꑱ
				da++;
				if(9 < (da & 0x0f)){			// X𒴂
					da &= 0xf0;
					da += 0x10;
				}
				if(syo == 1){					// ̌
					if(0x30 < da){
						da = 1;
					}
				}else if(gRTC_mon == 0x02){
					if(uru == 1){				// Q 邤N
						if(0x29 < da){
							da = 1;
						}
					}else{						// Q N
						if(0x28 < da){
							da = 1;
						}
					}
				}else{							// ̌
					if(0x31 < da){
						da = 1;
					}
				}
			}else if(ucr == SWR_OFF_SHORT){		// Zꂽ
				if(gRTC_day == da){				// lύXĂȂ
					r = 0;
				}else{
					gRTC_day = da;
					rtc_write();				// RTC 
				}
				break;							// I
			}
		}

		if (proc_no_key_timeup(0) == 0xff){		// ԃL[͂
			r = 0xff;
			break;
		}
	}

	return(r);
}
//*****************************************
// 킹
// ߂l@OFɂqsbɃANZXł
// @@@@PFɂqsbɃANZXłȂ
// @OFԑ얳
//*****************************************
unsigned char	proc_set_hour(void)
{
	unsigned char	blk_stat=0;
	unsigned char	ho;
	unsigned char	ucr;
	unsigned char	r=0;

	ho = gRTC_hou;
	set_7seg_data(ho >> 4,0,0);
	set_7seg_data(ho & 0x0f,0,1);
	set_7seg_data(gRTC_min >> 4,1,2);
	set_7seg_data(gRTC_min & 0x0f,0,3);

	tim_10ms_sblink = 0;						// _
	while(1){
		if(tim_10ms_sblink == 0){				// _Ń^C~O̎
			tim_10ms_sblink = 50;
			if(blk_stat==0){					// ł̎
				blk_stat = 1;
				set_7seg_data(16,0,0);
				set_7seg_data(16,0,1);
			}else{
				blk_stat = 0;
				set_7seg_data(ho >> 4,0,0);
				set_7seg_data(ho & 0x0f,0,1);
			}
		}

		ucr = get_action_sw();					// XCb`Ԃ
		ucr = ucr & 0x0f;						// EXCb`ɂ
		if(ucr != 0){							// EANVL鎞
			proc_no_key_timeup(1);				// L[͂L
			if(ucr == SWR_ON_CONTINUE){			// ꑱ
				ho++;
				if(9 < (ho & 0x0f)){			// X𒴂
					ho &= 0xf0;
					ho += 0x10;
				}
				if(0x23 < ho){
					ho = 0;
				}
			}else if(ucr == SWR_OFF_SHORT){		// Zꂽ
				if(gRTC_hou == ho){				// lύXĂȂ
					r = 0;
				}else{
					gRTC_hou = ho;
					rtc_write();				// RTC 
				}
				break;
			}
		}

		if (proc_no_key_timeup(0) == 0xff){		// ԃL[͂
			r = 0xff;
			break;
		}
	}

	return(r);
}
//*****************************************
// 킹
// ߂l@OFɂqsbɃANZXł
// @@@@PFɂqsbɃANZXłȂ
// @OFԑ얳
//*****************************************
unsigned char	proc_set_minute(void)
{
	unsigned char	blk_stat=0;
	unsigned char	mi;
	unsigned char	ucr;
	unsigned char	r=0;

	mi = gRTC_min;
	set_7seg_data(gRTC_hou >> 4,0,0);
	set_7seg_data(gRTC_hou & 0x0f,0,1);
	set_7seg_data(mi >> 4,0,2);
	set_7seg_data(mi & 0x0f,1,3);

	tim_10ms_sblink = 0;						// _
	while(1){
		if(tim_10ms_sblink == 0){				// _Ń^C~O̎
			tim_10ms_sblink = 50;
			if(blk_stat==0){					// ł̎
				blk_stat = 1;
				set_7seg_data(16,0,2);
				set_7seg_data(16,1,3);
			}else{
				blk_stat = 0;
				set_7seg_data(mi >> 4,0,2);
				set_7seg_data(mi & 0x0f,1,3);
			}
		}

		ucr = get_action_sw();					// XCb`Ԃ
		ucr = ucr & 0x0f;						// EXCb`ɂ
		if(ucr != 0){							// EANVL鎞
			proc_no_key_timeup(1);				// L[͂L
			if(ucr == SWR_ON_CONTINUE){			// ꑱ
				mi++;
				if(9 < (mi & 0x0f)){			// X𒴂
					mi &= 0xf0;
					mi += 0x10;
				}
				if(0x59 < mi){
					mi = 0;
				}
			}else if(ucr == SWR_OFF_SHORT){		// Zꂽ
				if(gRTC_min == mi){				// lύXĂȂ
					r = 0;
				}else{
					gRTC_min = mi;
					gRTC_sec = 0;
					rtc_write();				// RTC 
				}
				break;
			}
		}

		if (proc_no_key_timeup(0) == 0xff){		// ԃL[͂
			r = 0xff;
			break;
		}
	}

	return(r);
}
//*****************************************
// v킹
// ߂l@OFɂqsbɃANZXł
// @@@@PFɂqsbɃANZXłȂ
// @OFԑ얳
//*****************************************
unsigned char	proc_set_rtc(void)
{
	unsigned char	r;

	rtc_read();									// RTC ǂݏo

	gRTC_mon &= 0x1f;							// 
	gRTC_day &= 0x3f;							// 
	gRTC_hou &= 0x3f;							// 
	gRTC_min &= 0x7f;							// 
	gRTC_sec &= 0x7f;							// b

	r = proc_set_year();						// N킹
	if(r == 0){
		r = proc_set_month();					// 킹
		if(r == 0){
			r = proc_set_day();					// 킹
			if(r == 0){
				r = proc_set_hour();			// 킹
				if(r == 0){
					r = proc_set_minute();		// 킹
				}
			}
		}
	}
	return(r);
}

//*****************************************
// INT0 Interrupt
//  Start Bit Falling Edge Sence
// 	qsb@rpvM
//  \߁@qsb̏ݒłPbo͂ɐݒ肷邱
//*****************************************
ISR (INT0_vect)
{
	gCount_sqw++;								// rpvM̃JEg@{P
	rtc_1s_flg++;								// qsbĂ鎞̂Pboߗp

	if(tim_1s_no_key_timeup != 0){				// L[͖^CvAbvpPbPʃ^C}[
		tim_1s_no_key_timeup--;
	}
}
